;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "ecb" "20260518.524"
  "A code browser for Emacs."
  ()
  :url "https://github.com/ecb-org/ecb"
  :commit "ac7c585be4de7ba085ec91ef580361711a7f8c54"
  :revdesc "ac7c585be4de"
  :keywords '("browser" "code" "programming" "tools")
  :authors '(("Jesper Nordenberg" . "mayhem@home.se")
             ("Klaus Berndl" . "klaus.berndl@sdm.de")
             ("Kevin A. Burton" . "burton@openprivacy.org"))
  :maintainers '(("Klaus Berndl" . "klaus.berndl@sdm.de")))
